@@ -1,5 +1,5 @@
 /*
- * @(#)Proxy.java	1.20 04/04/20
+ * @(#)Proxy.java	1.21 05/09/15
  *
  * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
@@ -12,7 +12,9 @@ import java.lang.ref.WeakReference;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Map;
+import java.util.Set;
 import java.util.WeakHashMap;
 import sun.misc.ProxyGenerator;
 
@@ -195,7 +197,7 @@ import sun.misc.ProxyGenerator;
  * successfully by the <code>invoke</code> method.
  *
  * @author	Peter Jones
- * @version	1.20, 04/04/20
+ * @version	1.21, 05/09/15
  * @see		InvocationHandler
  * @since	1.3
  */
@@ -325,11 +327,17 @@ public class Proxy implements java.io.Serializable {
                                          Class<?>... interfaces)
 	throws IllegalArgumentException
     {
+	if (interfaces.length > 65535) {
+	    throw new IllegalArgumentException("interface limit exceeded");
+	}
+
 	Class proxyClass = null;
 
 	/* collect interface names to use as key for proxy class cache */
 	String[] interfaceNames = new String[interfaces.length];
 
+	Set interfaceSet = new HashSet();	// for detecting duplicates
+
 	for (int i = 0; i < interfaces.length; i++) {
 	    /*
 	     * Verify that the class loader resolves the name of this
@@ -355,6 +363,15 @@ public class Proxy implements java.io.Serializable {
 		    interfaceClass.getName() + " is not an interface");
 	    }
 
+	    /*
+	     * Verify that this interface is not a duplicate.
+	     */
+	    if (interfaceSet.contains(interfaceClass)) {
+		throw new IllegalArgumentException(
+		    "repeated interface: " + interfaceClass.getName());
+	    }
+	    interfaceSet.add(interfaceClass);
+
 	    interfaceNames[i] = interfaceName;
 	}
 
