@@ -1,5 +1,5 @@
 /*
- * @(#)GTKFileChooserUI.java	1.29 04/09/15
+ * @(#)GTKFileChooserUI.java	1.30 05/06/08
  *
  * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
@@ -259,6 +259,10 @@ class GTKFileChooserUI extends SynthFileChooserUI {
 	    } catch (IOException ioe) {
 		setDirectoryName(((File)e.getNewValue()).getAbsolutePath());
 	    }
+	    if ((getFileChooser().getFileSelectionMode() == JFileChooser.DIRECTORIES_ONLY) &&
+!getFileChooser().isMultiSelectionEnabled()) {
+		setFileName(pathField.getText());
+	    }
 	    directoryComboBoxModel.addItem(currentDirectory);
             directoryListModel.directoryChanged();
 	}
