@@ -2446,9 +2446,16 @@ public class XSLTErrorResources extends ListResourceBundle
    *
    * @return The int to message lookup table.
    */
-  public Object[][] getContents()
-  {
-    return contents;
+  protected Object[][] getContents() {
+      // return a copy of contents; in theory we want a deep clone
+      // of contents, but since it only contains (immutable) Strings,
+      // this shallow copy is sufficient
+      Object[][] commonCopy = new Object[contents.length][2];
+      for (int i = 0; i < contents.length; i++) {
+          commonCopy[i][0] = contents[i][0];
+          commonCopy[i][1] = contents[i][1];
+      }
+      return commonCopy;
   }
 
   /**
