@@ -1,5 +1,5 @@
 /*
- * @(#)TTYResources_ja.java	1.11 04/04/05
+ * @(#)TTYResources_ja.java	1.12 05/09/01
  *
  * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
@@ -15,14 +15,19 @@ package com.sun.tools.example.debug.tty;
  * <li> com.sun.tools.example.debug.tty
  * </ol>
  *
- * @version @(#)TTYResources_ja.java	1.11 04/04/05
+ * @version @(#)TTYResources_ja.java	1.12 05/09/01
  */
 public class TTYResources_ja extends java.util.ListResourceBundle {
 
     /**
+     * Returns the contents of this <code>ResourceBundle</code>.
      *
+     * <p>
+     *
+     * @return the contents of this <code>ResourceBundle</code>.
      */
-    private static final Object[][] contents = {
+    public Object[][] getContents() {
+        return new Object[][] {
         // NOTE: The value strings in this file containing "{0}" are
         //       processed by the java.text.MessageFormat class.  Any
         //       single quotes appearing in these strings need to be
@@ -409,16 +414,6 @@ public class TTYResources_ja extends java.util.ListResourceBundle {
              "\n" +
              "\u30b3\u30de\u30f3\u30c9\u30d8\u30eb\u30d7\u3092\u8868\u793a\u3059\u308b\u306b\u306f\u3001{0} \u30d7\u30ed\u30f3\u30d7\u30c8\u306b ''help'' \u3068\u5165\u529b\u3057\u307e\u3059\u3002"},
         // END OF MATERIAL TO LOCALIZE
-    };
-
-    /**
-     * Returns the contents of this <code>ResourceBundle</code>.
-     *
-     * <p>
-     *
-     * @return the contents of this <code>ResourceBundle</code>.
-     */
-    public Object[][] getContents() {
-        return contents;
+        };
     }
 }
