@@ -1,5 +1,5 @@
 /*
- * @(#)BasicTableUI.java	1.142 05/03/03
+ * @(#)BasicTableUI.java	1.148 06/04/18
  *
  * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
@@ -20,6 +20,9 @@ import javax.swing.event.*;
 import javax.swing.plaf.*;
 import javax.swing.text.*;
 import javax.swing.table.*;
+import javax.swing.plaf.basic.DragRecognitionSupport.BeforeDrag;
+import com.sun.java.swing.SwingUtilities2;
+import static com.sun.java.swing.SwingUtilities2.DRAG_FIX;
 
 
 import java.beans.PropertyChangeEvent;
@@ -31,8 +34,9 @@ import sun.swing.UIAction;
 /**
  * BasicTableUI implementation
  *
- * @version 1.142 03/03/05
+ * @version 1.148 04/18/06
  * @author Philip Milne
+ * @author Shannon Hickey (improved drag recognition)
  */
 public class BasicTableUI extends TableUI
 {
@@ -52,6 +56,11 @@ public class BasicTableUI extends TableUI
 
     private Handler handler;
 
+    /**
+     * Local cache of Table's client property "Table.isFileList"
+     */
+    private boolean isFileList = false;
+
 //
 //  Helper class for keyboard actions
 //
@@ -277,6 +286,22 @@ public class BasicTableUI extends TableUI
                 stayInSelection = true;
             }
 
+            // the algorithm below isn't prepared to deal with -1 lead/anchor
+            // so massage appropriately here first
+            if (dy == 1 && leadColumn == -1) {
+                leadColumn = minX;
+                leadRow = -1;
+            } else if (dx == 1 && leadRow == -1) {
+                leadRow = minY;
+                leadColumn = -1;
+            } else if (dy == -1 && leadColumn == -1) {
+                leadColumn = maxX;
+                leadRow = maxY + 1;
+            } else if (dx == -1 && leadRow == -1) {
+                leadRow = maxY;
+                leadColumn = maxX + 1;
+            }
+
             // In cases where the lead is not within the search range,
             // we need to bring it within one cell for the the search
             // to work properly. Check these here.
@@ -336,10 +361,10 @@ public class BasicTableUI extends TableUI
             JTable table = (JTable)e.getSource();
 
             ListSelectionModel rsm = table.getSelectionModel();
-            leadRow = rsm.getLeadSelectionIndex();
+            leadRow = getAdjustedLead(table, true, rsm);
 
             ListSelectionModel csm = table.getColumnModel().getSelectionModel();
-            leadColumn =   csm.getLeadSelectionIndex();
+            leadColumn = getAdjustedLead(table, false, csm);
 
             if (!table.getComponentOrientation().isLeftToRight()) {
                 if (key == SCROLL_LEFT_CHANGE_SELECTION ||
@@ -387,27 +412,24 @@ public class BasicTableUI extends TableUI
     
                     Dimension delta = table.getParent().getSize();
 
-                    int start = (vertically) ? rsm.getLeadSelectionIndex() :
-                                               csm.getLeadSelectionIndex();
-
                     if (vertically) {
-                        Rectangle r = table.getCellRect(start, 0, true);
+                        Rectangle r = table.getCellRect(leadRow, 0, true);
                         r.y += forwards ? delta.height : -delta.height;
                         this.dx = 0;
                         int newRow = table.rowAtPoint(r.getLocation());
                         if (newRow == -1 && forwards) {
                             newRow = table.getRowCount();
                         }
-                        this.dy = newRow - start;
+                        this.dy = newRow - leadRow;
                     }
                     else {
-                        Rectangle r = table.getCellRect(0, start, true);
+                        Rectangle r = table.getCellRect(0, leadColumn, true);
                         r.x += forwards ? delta.width : -delta.width;
                         int newColumn = table.columnAtPoint(r.getLocation());
                         if (newColumn == -1 && forwards) {
                             newColumn = table.getColumnCount();
                         }
-                        this.dx = newColumn - start;
+                        this.dx = newColumn - leadColumn;
                         this.dy = 0;
                     }
                 }
@@ -481,10 +503,20 @@ public class BasicTableUI extends TableUI
                         // casting should be safe since the action is only enabled
                         // for DefaultListSelectionModel
                         ((DefaultListSelectionModel)rsm).moveLeadSelectionIndex(leadRow);
+                        if (getAdjustedLead(table, false, csm) == -1
+                                && table.getColumnCount() > 0) {
+
+                            ((DefaultListSelectionModel)csm).moveLeadSelectionIndex(0);
+                        }
                     } else {
                         // casting should be safe since the action is only enabled
                         // for DefaultListSelectionModel
                         ((DefaultListSelectionModel)csm).moveLeadSelectionIndex(leadColumn);
+                        if (getAdjustedLead(table, true, rsm) == -1
+                                && table.getRowCount() > 0) {
+
+                            ((DefaultListSelectionModel)rsm).moveLeadSelectionIndex(0);
+                        }
                     }
 
                     Rectangle cellRect = table.getCellRect(leadRow, leadColumn, false);
@@ -496,6 +528,11 @@ public class BasicTableUI extends TableUI
                     table.changeSelection(leadRow, leadColumn, false, extend);
                 }
                 else {
+                    if (table.getRowCount() <= 0 || table.getColumnCount() <= 0) {
+                        // bail - don't try to move selection on an empty table
+                        return;
+                    }
+
                     if (moveWithinSelectedRange(table, dx, dy, rsm, csm)) {
                         // this is the only way we have to set both the lead
                         // and the anchor without changing the selection
@@ -629,10 +666,8 @@ public class BasicTableUI extends TableUI
                 // into the cell and begin editing. In both of these cases
                 // this action will be disabled.
                 JTable table = (JTable)sender;
-                int leadRow = table.getSelectionModel().
-                                  getLeadSelectionIndex();
-                int leadCol = table.getColumnModel().getSelectionModel().
-                                  getLeadSelectionIndex();
+                int leadRow = getAdjustedLead(table, true);
+                int leadCol = getAdjustedLead(table, false);
                 return !(table.isEditing() || table.isCellSelected(leadRow, leadCol));
             }
 
@@ -743,14 +778,12 @@ public class BasicTableUI extends TableUI
 
         // FocusListener
         private void repaintLeadCell( ) {
-	    int rc = table.getRowCount();
-	    int cc = table.getColumnCount();
-            int lr = table.getSelectionModel().getLeadSelectionIndex();
-            int lc = table.getColumnModel().getSelectionModel().
-                    getLeadSelectionIndex();
-	    if (lr < 0 || lr >= rc || lc < 0 || lc >= cc) {
-		return;
-	    }
+            int lr = getAdjustedLead(table, true);
+            int lc = getAdjustedLead(table, false);
+
+            if (lr < 0 || lc < 0) {
+                return;
+            }
 
             Rectangle dirtyRect = table.getCellRect(lr, lc, false);
             table.repaint(dirtyRect);
@@ -797,9 +830,8 @@ public class BasicTableUI extends TableUI
                 return;
             }
 
-            int leadRow = table.getSelectionModel().getLeadSelectionIndex();
-            int leadColumn = table.getColumnModel().getSelectionModel().
-                    getLeadSelectionIndex();
+            int leadRow = getAdjustedLead(table, true);
+            int leadColumn = getAdjustedLead(table, false);
             if (leadRow != -1 && leadColumn != -1 && !table.isEditing()) {
                 if (!table.editCellAt(leadRow, leadColumn)) {
                     return;
@@ -843,21 +875,23 @@ public class BasicTableUI extends TableUI
 
         // Component receiving mouse events during editing.
         // May not be editorComponent.
-        private Component dispatchComponent;
+        protected Component dispatchComponent;
 	private boolean selectedOnPress;
 
         public void mouseClicked(MouseEvent e) {}
 
-        private void setDispatchComponent(MouseEvent e) {
+        protected void setDispatchComponent(MouseEvent e) {
             Component editorComponent = table.getEditorComponent();
             Point p = e.getPoint();
             Point p2 = SwingUtilities.convertPoint(table, p, editorComponent);
             dispatchComponent =
                     SwingUtilities.getDeepestComponentAt(editorComponent,
                             p2.x, p2.y);
+            SwingUtilities2.setSkipClickCount(dispatchComponent,
+                                              e.getClickCount() - 1);
         }
 
-        private boolean repostEvent(MouseEvent e) {
+        protected boolean repostEvent(MouseEvent e) {
 	    // Check for isEditing() in case another event has
 	    // caused the editor to be removed. See bug #4306499.
             if (dispatchComponent == null || !table.isEditing()) {
@@ -869,15 +903,14 @@ public class BasicTableUI extends TableUI
             return true;
         }
 
-        private void setValueIsAdjusting(boolean flag) {
+        protected void setValueIsAdjusting(boolean flag) {
             table.getSelectionModel().setValueIsAdjusting(flag);
             table.getColumnModel().getSelectionModel().
                     setValueIsAdjusting(flag);
         }
 
-	private boolean shouldIgnore(MouseEvent e) {
-	    return e.isConsumed() || (!(SwingUtilities.isLeftMouseButton(e) &&
-                        table.isEnabled()));
+	private boolean shouldIgnore0(MouseEvent e) {
+            return e.isConsumed() || SwingUtilities2.shouldIgnore(e, table);
 	}
 
         public void mousePressed(MouseEvent e) {
@@ -890,7 +923,7 @@ public class BasicTableUI extends TableUI
 	}
 
 	void adjustFocusAndSelection(MouseEvent e) {
-	    if (shouldIgnore(e)) {
+	    if (shouldIgnore0(e)) {
 	        return;
 	    }
 
@@ -923,8 +956,8 @@ public class BasicTableUI extends TableUI
                 setDispatchComponent(e);
                 repostEvent(e);
             }
-	    else if (table.isRequestFocusEnabled()) {
-                table.requestFocus();
+	    else {
+                SwingUtilities2.adjustFocus(table);
 	    }
 
             CellEditor editor = table.getCellEditor();
@@ -932,38 +965,57 @@ public class BasicTableUI extends TableUI
 		boolean adjusting = (e.getID() == MouseEvent.MOUSE_PRESSED) ?
                         true : false;
                 setValueIsAdjusting(adjusting);
-                boolean ctrl = e.isControlDown();
+                makeSelectionChange(row, column, e);
+	    }
+        }
 
-                // Apply the selection state of the anchor to all cells between it and the
-                // current cell, and then select the current cell.
-                // For mustang, where API changes are allowed, this logic will moved to
-                // JTable.changeSelection()
-                if (ctrl && e.isShiftDown()) {
-                    ListSelectionModel rm = table.getSelectionModel();
-                    ListSelectionModel cm = table.getColumnModel().getSelectionModel();
-                    int anchorRow = rm.getAnchorSelectionIndex();
-                    int anchorCol = cm.getAnchorSelectionIndex();
+        protected void makeSelectionChange(int row, int column, MouseEvent e) {
+            boolean ctrl = e.isControlDown();
 
-                    if (table.isCellSelected(anchorRow, anchorCol)) {
-                        rm.addSelectionInterval(anchorRow, row);
-                        cm.addSelectionInterval(anchorCol, column);
-                    } else {
-                        rm.removeSelectionInterval(anchorRow, row);
+            // Apply the selection state of the anchor to all cells between it and the
+            // current cell, and then select the current cell.
+            // For mustang, where API changes are allowed, this logic will moved to
+            // JTable.changeSelection()
+            if (ctrl && e.isShiftDown()) {
+                ListSelectionModel rm = table.getSelectionModel();
+                ListSelectionModel cm = table.getColumnModel().getSelectionModel();
+                int anchorRow = rm.getAnchorSelectionIndex();
+                int anchorCol = cm.getAnchorSelectionIndex();
+
+                boolean anchorSelected = true;
+                if (anchorRow == -1 || anchorRow >= table.getRowCount()) {
+                    anchorRow = 0;
+                    anchorSelected = false;
+                }
+
+                if (anchorCol == -1 || anchorCol >= table.getColumnCount()) {
+                    anchorCol = 0;
+                    anchorSelected = false;
+                }
+
+                if (anchorSelected && table.isCellSelected(anchorRow, anchorCol)) {
+                    rm.addSelectionInterval(anchorRow, row);
+                    cm.addSelectionInterval(anchorCol, column);
+                } else {
+                    rm.removeSelectionInterval(anchorRow, row);
+                    cm.removeSelectionInterval(anchorCol, column);
+
+                    // This is only to match the windows explorer behavior in JFileChooser
+                    if (isFileList) {
                         rm.addSelectionInterval(row, row);
                         rm.setAnchorSelectionIndex(anchorRow);
-                        cm.removeSelectionInterval(anchorCol, column);
                         cm.addSelectionInterval(column, column);
                         cm.setAnchorSelectionIndex(anchorCol);
                     }
-                } else {
-                    table.changeSelection(row, column, ctrl, !ctrl && e.isShiftDown());
                 }
-	    }
+            } else {
+                table.changeSelection(row, column, ctrl, !ctrl && e.isShiftDown());
+            }
         }
 
         public void mouseReleased(MouseEvent e) {
 	    if (selectedOnPress) {
-		if (shouldIgnore(e)) {
+		if (shouldIgnore0(e)) {
 		    return;
 		}
 
@@ -983,31 +1035,79 @@ public class BasicTableUI extends TableUI
         public void mouseMoved(MouseEvent e) {}
 
         public void mouseDragged(MouseEvent e) {
-	    if (shouldIgnore(e)) {
+	    if (shouldIgnore0(e)) {
 	        return;
 	    }
 
+            mouseDraggedImpl(e);
+        }
+
+        protected void mouseDraggedImpl(MouseEvent e) {
             repostEvent(e);
 
-            CellEditor editor = table.getCellEditor();
-            if (editor == null || editor.shouldSelectCell(e)) {
-                Point p = e.getPoint();
-                int row = table.rowAtPoint(p);
-                int column = table.columnAtPoint(p);
-	        // The autoscroller can generate drag events outside the
-                // table's range.
-                if ((column == -1) || (row == -1)) {
-                    return;
+            // Check isFileList:
+            // Until we support drag-selection, dragging should not change
+            // the selection (act like single-select).
+            if (isFileList || table.isEditing()) {
+                return;
+            }
+
+            Point p = e.getPoint();
+            int row = table.rowAtPoint(p);
+            int column = table.columnAtPoint(p);
+	    // The autoscroller can generate drag events outside the
+            // table's range.
+            if ((column == -1) || (row == -1)) {
+                return;
+            }
+
+            if (e.isControlDown()) {
+                ListSelectionModel cm = table.getColumnModel().getSelectionModel();
+                ListSelectionModel rm = table.getSelectionModel();
+                int colAnchor = cm.getAnchorSelectionIndex();
+                int rowAnchor = rm.getAnchorSelectionIndex();
+
+                boolean selected = true;
+
+                if (rowAnchor == -1 || rowAnchor >= table.getRowCount()) {
+                    rowAnchor = 0;
+                    selected = false;
                 }
-                // Fix for 4835633
-                // Until we support drag-selection, dragging should not change
-                // the selection (act like single-select).
-                Object bySize = table.getClientProperty("Table.isFileList");
-                if (bySize instanceof Boolean &&
-                    ((Boolean)bySize).booleanValue()) {
-                    return;
+
+                if (colAnchor == -1 || colAnchor >= table.getColumnCount()) {
+                    colAnchor = 0;
+                    selected = false;
                 }
-	        table.changeSelection(row, column, false, true);
+
+                selected = selected && table.isCellSelected(rowAnchor, colAnchor);
+
+                changeSelectionModel(cm, colAnchor, selected, column);
+                changeSelectionModel(rm, rowAnchor, selected, row);
+
+                // From JTable.changeSelection():
+                // Scroll after changing the selection as blit scrolling is immediate,
+                // so that if we cause the repaint after the scroll we end up painting
+                // everything!
+                if (table.getAutoscrolls()) {
+                    Rectangle cellRect = table.getCellRect(row, column, false);
+                    if (cellRect != null) {
+                        table.scrollRectToVisible(cellRect);
+                    }
+                }
+            } else {
+                table.changeSelection(row, column, false, true);
+            }
+        }
+
+        private void changeSelectionModel(ListSelectionModel sm,
+                                          int anchorIndex,
+                                          boolean anchorSelected,
+                                          int index) {
+
+            if (anchorSelected) {
+                sm.addSelectionInterval(anchorIndex, index);
+            } else {
+                sm.removeSelectionInterval(anchorIndex, index);
             }
         }
 
@@ -1036,11 +1136,314 @@ public class BasicTableUI extends TableUI
                         // should not happen... swing drop target is multicast
                     }
                 }
+            } else if ("Table.isFileList" == changeName) {
+                isFileList = Boolean.TRUE.equals(table.getClientProperty("Table.isFileList"));
+                table.revalidate();
+                table.repaint();
             }
 	}
     }
 
 
+    private class DragFixHandler extends Handler implements ListSelectionListener,
+                                                            ActionListener,
+                                                            BeforeDrag {
+
+        // The row and column where the press occurred and the
+        // press event itself
+        private int pressedRow;
+        private int pressedCol;
+        private MouseEvent pressedEvent;
+
+        // Whether or not the mouse press (which is being considered as part
+        // of a drag sequence) also caused the selection change to be fully
+        // processed.
+        private boolean dragPressDidSelection;
+        
+        // Set to true when a drag gesture has been fully recognized and DnD
+        // begins. Use this to ignore further mouse events which could be
+        // delivered if DnD is cancelled (via ESCAPE for example)
+        private boolean dragStarted;
+
+        // Whether or not we should start the editing timer on release
+        private boolean shouldStartTimer;
+
+        // To cache the return value of pointOutsidePrefSize since we use
+        // it multiple times.
+        private boolean outsidePrefSize;
+
+        // Used to delay the start of editing.
+        private Timer timer = null;
+
+        private boolean canStartDrag() {
+            if (pressedRow == -1 || pressedCol == -1) {
+                return false;
+            }
+
+            if (isFileList) {
+                return !outsidePrefSize;
+            }
+
+            // if this is a single selection table
+            if ((table.getSelectionModel().getSelectionMode() ==
+                     ListSelectionModel.SINGLE_SELECTION) &&
+                (table.getColumnModel().getSelectionModel().getSelectionMode() ==
+                     ListSelectionModel.SINGLE_SELECTION)) {
+
+                return true;
+            }
+
+            return table.isCellSelected(pressedRow, pressedCol);
+        }
+
+        public void mousePressed(MouseEvent e) {
+            if (SwingUtilities2.shouldIgnore(e, table)) {
+                return;
+            }
+
+            if (table.isEditing() && !table.getCellEditor().stopCellEditing()) {
+                Component editorComponent = table.getEditorComponent();
+                if (editorComponent != null && !editorComponent.hasFocus()) {
+                    BasicLookAndFeel.compositeRequestFocus(editorComponent);
+                }
+                return;
+            }
+
+            Point p = e.getPoint();
+            pressedRow = table.rowAtPoint(p);
+            pressedCol = table.columnAtPoint(p);
+            outsidePrefSize = pointOutsidePrefSize(table, pressedRow, pressedCol, p);
+
+            if (isFileList) {
+                shouldStartTimer =
+                    table.isCellSelected(pressedRow, pressedCol) &&
+                    !e.isShiftDown() &&
+                    !e.isControlDown() &&
+                    !outsidePrefSize;
+            }
+
+            if (table.getDragEnabled()) {
+                mousePressedDND(e);
+            } else {
+                SwingUtilities2.adjustFocus(table);
+                if (!isFileList) {
+                    setValueIsAdjusting(true);
+                }
+                adjustSelection(e);
+            }
+        }
+
+        private void mousePressedDND(MouseEvent e) {
+            pressedEvent = e;
+            boolean grabFocus = true;
+            dragStarted = false;
+
+            if (canStartDrag() && DragRecognitionSupport.mousePressed(e)) {
+
+                dragPressDidSelection = false;
+
+                if (e.isControlDown() && isFileList) {
+                    // do nothing for control - will be handled on release
+                    // or when drag starts
+                    return;
+                } else if (!e.isShiftDown() && table.isCellSelected(pressedRow, pressedCol)) {
+                    // clicking on something that's already selected
+                    // and need to make it the lead now
+                    table.getSelectionModel().addSelectionInterval(pressedRow,
+                                                                   pressedRow);
+                    table.getColumnModel().getSelectionModel().
+                        addSelectionInterval(pressedCol, pressedCol);
+
+                    return;
+                }
+
+                dragPressDidSelection = true;
+
+                // could be a drag initiating event - don't grab focus
+                grabFocus = false;
+            } else if (!isFileList) {
+                // When drag can't happen, mouse drags might change the selection in the table
+                // so we want the isAdjusting flag to be set
+                setValueIsAdjusting(true);
+            }
+
+            if (grabFocus) {
+                SwingUtilities2.adjustFocus(table);
+            }
+
+            adjustSelection(e);
+        }
+
+        private void adjustSelection(MouseEvent e) {
+            // Fix for 4835633
+            if (outsidePrefSize) {
+                // If shift is down in multi-select, we should just return.
+                // For single select or non-shift-click, clear the selection
+                if (e.getID() ==  MouseEvent.MOUSE_PRESSED &&
+                    (!e.isShiftDown() ||
+                     table.getSelectionModel().getSelectionMode() ==
+                     ListSelectionModel.SINGLE_SELECTION)) {
+                    table.clearSelection();
+                    TableCellEditor tce = table.getCellEditor();
+                    if (tce != null) {
+                        tce.stopCellEditing();
+                    }
+                }
+                return;
+            }
+            // The autoscroller can generate drag events outside the
+            // table's range.
+            if ((pressedCol == -1) || (pressedRow == -1)) {
+                return;
+            }
+
+            boolean dragEnabled = table.getDragEnabled();
+
+            if (!dragEnabled && !isFileList && table.editCellAt(pressedRow, pressedCol, e)) {
+                setDispatchComponent(e);
+                repostEvent(e);
+            }
+
+            CellEditor editor = table.getCellEditor();
+            if (dragEnabled || editor == null || editor.shouldSelectCell(e)) {
+                makeSelectionChange(pressedRow, pressedCol, e);
+            }
+        }
+
+        public void valueChanged(ListSelectionEvent e) {
+            if (timer != null) {
+                timer.stop();
+                timer = null;
+            }
+        }
+
+        public void actionPerformed(ActionEvent ae) {
+            table.editCellAt(pressedRow, pressedCol, null);
+            Component editorComponent = table.getEditorComponent();
+            if (editorComponent != null && !editorComponent.hasFocus()) {
+                BasicLookAndFeel.compositeRequestFocus(editorComponent);
+            }
+            return;
+        }
+
+        private void maybeStartTimer() {
+            if (!shouldStartTimer) {
+                return;
+            }
+
+            if (timer == null) {
+                timer = new Timer(1200, this);
+                timer.setRepeats(false);
+            }
+
+            timer.start();
+        }
+
+        public void mouseReleased(MouseEvent e) {
+            if (SwingUtilities2.shouldIgnore(e, table)) {
+                return;
+            }
+
+            if (table.getDragEnabled()) {
+                mouseReleasedDND(e);
+            } else {
+                if (isFileList) {
+                    maybeStartTimer();
+                }
+            }
+
+            pressedEvent = null;
+            repostEvent(e);
+            dispatchComponent = null;
+            setValueIsAdjusting(false);
+        }
+
+        private void mouseReleasedDND(MouseEvent e) {
+            MouseEvent me = DragRecognitionSupport.mouseReleased(e);
+            if (me != null) {
+                SwingUtilities2.adjustFocus(table);
+                if (!dragPressDidSelection) {
+                    adjustSelection(me);
+                }
+            }
+
+            if (!dragStarted) {
+                if (isFileList) {
+                    maybeStartTimer();
+                    return;
+                }
+                
+                Point p = e.getPoint();
+
+                if (pressedEvent != null &&
+                        table.rowAtPoint(p) == pressedRow &&
+                        table.columnAtPoint(p) == pressedCol &&
+                        table.editCellAt(pressedRow, pressedCol, pressedEvent)) {
+
+                    setDispatchComponent(pressedEvent);
+                    repostEvent(pressedEvent);
+
+                    // This may appear completely odd, but must be done for backward
+                    // compatibility reasons. Developers have been known to rely on
+                    // a call to shouldSelectCell after editing has begun.
+                    CellEditor ce = table.getCellEditor();
+                    if (ce != null) {
+                        ce.shouldSelectCell(pressedEvent);
+                    }
+                }
+            }
+        }
+
+        public void dragStarting(MouseEvent me) {
+            dragStarted = true;
+
+            if (me.isControlDown() && isFileList) {
+                table.getSelectionModel().addSelectionInterval(pressedRow,
+                                                               pressedRow);
+                table.getColumnModel().getSelectionModel().
+                    addSelectionInterval(pressedCol, pressedCol);
+            }
+
+            pressedEvent = null;
+        }
+
+        public void mouseDragged(MouseEvent e) {
+            if (SwingUtilities2.shouldIgnore(e, table)) {
+                return;
+            }
+
+            if (table.getDragEnabled() &&
+                    (DragRecognitionSupport.mouseDragged(e, this) || dragStarted)) {
+
+                return;
+            }
+
+            mouseDraggedImpl(e);
+        }
+
+        public void propertyChange(PropertyChangeEvent event) {
+            super.propertyChange(event);
+
+            String changeName = event.getPropertyName();
+
+            if ("Table.isFileList" == changeName) {
+                if (isFileList) {
+                    table.getSelectionModel().addListSelectionListener(getDragFixHandler());
+                } else {
+                    table.getSelectionModel().removeListSelectionListener(getDragFixHandler());
+                    timer = null;
+                }
+            } else if ("selectionModel" == changeName) {
+                if (isFileList) {
+                    ListSelectionModel old = (ListSelectionModel)event.getOldValue();
+                    old.removeListSelectionListener(getDragFixHandler());
+                    table.getSelectionModel().addListSelectionListener(getDragFixHandler());
+                }
+            }
+        }
+    }
+
+
     /*
      * Returns true if the given point is outside the preferredSize of the
      * item at the given row of the table.  (Column must be 0).
@@ -1048,29 +1451,11 @@ public class BasicTableUI extends TableUI
      */
     private static boolean pointOutsidePrefSize(JTable table,
                                                 int row, int column, Point p) {
-        Object bySize = table.getClientProperty("Table.isFileList");
-        if (bySize instanceof Boolean && ((Boolean)bySize).booleanValue()) {
-            if (table.convertColumnIndexToModel(column) != 0 || row == -1) {
-                return true;
-            }
-            TableCellRenderer tcr = table.getCellRenderer(row, column);
-            Object value = table.getValueAt(row, column);
-            Component cell = tcr.getTableCellRendererComponent(table, value, false,
-                    false, row, column);
-            Dimension itemSize = cell.getPreferredSize();
-            Rectangle cellBounds = table.getCellRect(row, column, false);
-            cellBounds.width = itemSize.width;
-            cellBounds.height = itemSize.height;
-
-            // See if coords are inside
-            // ASSUME: mouse x,y will never be < cell's x,y
-            assert (p.x >= cellBounds.x && p.y >= cellBounds.y);
-            if (p.x > cellBounds.x + cellBounds.width ||
-                    p.y > cellBounds.y + cellBounds.height) {
-                return true;
-            }
+        if (!Boolean.TRUE.equals(table.getClientProperty("Table.isFileList"))) {
+            return false;
         }
-        return false;
+
+        return SwingUtilities2.pointOutsidePrefSize(table, row, column, p);
     }
 
 //
@@ -1079,11 +1464,17 @@ public class BasicTableUI extends TableUI
 
     private Handler getHandler() {
         if (handler == null) {
-            handler = new Handler();
+            handler = DRAG_FIX ? new DragFixHandler() : new Handler();
         }
         return handler;
     }
 
+    private DragFixHandler getDragFixHandler() {
+        // this only called by code that's enabled when DRAG_FIX is on
+        assert DRAG_FIX;
+        return (DragFixHandler)handler;
+    }
+
     /**
      * Creates the key listener for handling keyboard navigation in the JTable.
      */
@@ -1170,6 +1561,8 @@ public class BasicTableUI extends TableUI
                 LookAndFeel.installBorder((JScrollPane)parent, "Table.scrollPaneBorder");
             }
         }
+
+        isFileList = Boolean.TRUE.equals(table.getClientProperty("Table.isFileList"));
     }
 
     private void installDefaults2() {
@@ -1201,11 +1594,16 @@ public class BasicTableUI extends TableUI
 
         table.addFocusListener(focusListener);
         table.addKeyListener(keyListener);
-	table.addMouseListener(defaultDragRecognizer);
-	table.addMouseMotionListener(defaultDragRecognizer);
+        if (!DRAG_FIX) {
+            table.addMouseListener(defaultDragRecognizer);
+            table.addMouseMotionListener(defaultDragRecognizer);
+        }
         table.addMouseListener(mouseInputListener);
         table.addMouseMotionListener(mouseInputListener);
         table.addPropertyChangeListener(getHandler());
+        if (DRAG_FIX && isFileList) {
+            table.getSelectionModel().addListSelectionListener(getDragFixHandler());
+        }
     }
 
     /**
@@ -1351,11 +1749,16 @@ public class BasicTableUI extends TableUI
     protected void uninstallListeners() {
         table.removeFocusListener(focusListener);
         table.removeKeyListener(keyListener);
-	table.removeMouseListener(defaultDragRecognizer);
-	table.removeMouseMotionListener(defaultDragRecognizer);
+        if (!DRAG_FIX) {
+            table.removeMouseListener(defaultDragRecognizer);
+            table.removeMouseMotionListener(defaultDragRecognizer);
+        }
         table.removeMouseListener(mouseInputListener);
         table.removeMouseMotionListener(mouseInputListener);
         table.removePropertyChangeListener(getHandler());
+        if (DRAG_FIX && isFileList) {
+            table.getSelectionModel().removeListSelectionListener(getDragFixHandler());
+        }
 
         focusListener = null;
         keyListener = null;
@@ -1672,8 +2075,23 @@ public class BasicTableUI extends TableUI
         }
     }
 
+    private static int getAdjustedLead(JTable table,
+                                       boolean row,
+                                       ListSelectionModel model) {
 
-    private static final TableDragGestureRecognizer defaultDragRecognizer = new TableDragGestureRecognizer();
+        int index = model.getLeadSelectionIndex();
+        int compare = row ? table.getRowCount() : table.getColumnCount();
+        return index < compare ? index : -1;
+    }
+
+    private static int getAdjustedLead(JTable table, boolean row) {
+        return row ? getAdjustedLead(table, row, table.getSelectionModel())
+                   : getAdjustedLead(table, row, table.getColumnModel().getSelectionModel());
+    }
+
+
+    private static final TableDragGestureRecognizer defaultDragRecognizer =
+        DRAG_FIX ? null : new TableDragGestureRecognizer();
 
     /**
      * Drag gesture recognizer for JTable components
